from typing import List


class Token:
    def __init__(self, text: str, is_rejected=False) -> None:
        self.text = text
        self.__original = text
        self.is_rejected = is_rejected

    @property
    def original(self) -> str:
        return self.__original


class Document:
    def __init__(self, text: str, is_rejected=False, tokens=None) -> None:
        self.text = text
        self.__original = text
        self.is_rejected = is_rejected
        self.processed_text = ""
        if tokens is None:
            self.tokens: List[Token] = []

        self.dedup_lsh: List[str] = []

    @property
    def original(self) -> str:
        return self.__original

    def set_tokens(self, tokens: List[str]) -> None:
        self.tokens = [Token(token) for token in tokens]

    def get_tokens(self) -> List[str]:
        return [token.text for token in self.tokens]



from .core.composition import Compose
from .core.inspection import Inspector
from .core.models import Document, Token
from .filters import deduplication, document_filters, token_filters, tokenization

__all__ = [
    "Compose",
    "Inspector",
    "Document",
    "Token",
    "deduplication",
    "document_filters",
    "token_filters",
    "tokenization",
]



import logging

from hojichar.core.models import Document, Token


class Filter:
    """
    Base class for all filters.
    Document-level filters must inherit from this class.

    The definition of filter function is in `apply` method.
    If you define a new filter, you must define the method.
    When this class is called, apply the filter from string to string.

    If the filter create `Document.tokens` form `Document.text`, you
    must implement `tokenize` method.
    If the filter update `Document.text` by merging `Document.tokens`, you
    must implement `merge` method.
    Do not define a filter that changes both `Document.text` and `Document.token`
    to prevent unexpected behavior.

    If you apply the filter to tokens, you can use `TokenFilter` class.
    """

    def __init__(self, p: float = 1, *args, **kwargs):
        """
        Parameters
        ----------
        p : float, optional
            Probability that this filter will be applied. Default=1
        """
        self.name = self.__class__.__name__
        self.logger = logging.getLogger("hojichar.document_filters." + self.name)
        assert 0 <= p <= 1
        self.p = p

    def apply(self, document: Document) -> Document:
        """Definition of filter behavior.

        In this method, the filter will modify `document.text`, or
        set `document.is_rejected = True` to discard the document.

        Do not define a filter that changes both `document.text` and `document.token`

        Parameters
        ----------
        document : Document
            Input document

        Returns
        -------
        Document
            Processed Document
        """
        raise NotImplementedError(f"{self.__class__.__name__}.apply method is not defined")
        return document

    def filter_apply(self, document: Document) -> Document:
        if document.is_rejected:
            return document
        else:
            return self.apply(document)

    def __call__(self, text: str) -> str:
        document = Document(text)
        document = self.filter_apply(document)
        return document.text


class TokenFilter:
    """
    Base class for token-level filters.

    Token filters, which shuld be implemented in hojichar/filters/token_filters.py,
    must inherit from this class.
    """

    def __init__(self, p=1, *args, **kwargs):
        self.name = self.__class__.__name__
        self.logger = logging.getLogger("hojichar.token_filters." + self.name)
        assert 0 <= p <= 1
        self.p = p

    def apply(self, token: Token) -> Token:
        raise NotImplementedError(f"{self.__class__.__name__}.apply method is not defined")
        return token

    def filter_apply(self, document: Document) -> Document:
        if document.is_rejected:
            return document
        else:
            document.tokens = [
                self.apply(token) for token in document.tokens if not token.is_rejected
            ]
            return document

    def __call__(self, text: str) -> str:
        token = Token(text)
        token = self.apply(token)
        return token.text



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hojichar', 'hojichar.core', 'hojichar.dict', 'hojichar.filters']

package_data = \
{'': ['*']}

install_requires = \
['mmh3>=3.0.0,<4.0.0', 'numpy>=1.23.4,<2.0.0']

setup_kwargs = {
    'name': 'hojichar',
    'version': '0.3.0',
    'description': 'Text preprocessing management system.',
    'long_description': '# HojiChar\n[![PyPI version](https://badge.fury.io/py/hojichar.svg)](https://badge.fury.io/py/hojichar)\n[![Python Versions](https://img.shields.io/pypi/pyversions/hojichar.svg)](https://pypi.org/project/hojichar/)\n[![CI wowkflow](https://github.com/HojiChar/HojiChar/actions/workflows/ci.yaml/badge.svg)](https://github.com/HojiChar/HojiChar/actions/workflows/ci.yaml)\n[![codecov](https://codecov.io/gh/HojiChar/HojiChar/branch/main/graph/badge.svg?token=16928I9U9Y)](https://codecov.io/gh/HojiChar/HojiChar)\n\n## 概要\nHojiChar はテキストデータの前処理のためのPythonモジュールです. 言語モデル構築時などにコーパスを前処理する目的で開発されました。\n\n`hojichar.filters` で定義された、あるいはユーザーが定義したテキスト前処理フィルタを束ね、ひとつの前処理パイプラインとして構成することができように作られています。\n\nこの前処理パイプラインは、`torchvision.transforms` に着想を得て開発されました。\n\n\n## 使い方\n### インストール\n```\npip install hojichar\n```\n\n**Poetry によるローカルインストール**\n\n`python >= 3.8, poetry >= 1.2`\n\n```\nhttps://github.com/HojiChar/HojiChar.git\ncd HojiChar\npoetry install\n```\n\n\n### Rocket start\n`Compose` クラスを使ってフィルタを作成します.\n```Python\nfrom hojichar import Compose, document_filters\n\ncleaner = Compose([\n    document_filters.JSONLoader(key="text"),\n    document_filters.AcceptJapanese(),\n    document_filters.DocumentLengthFilter(min_doc_len=0,max_doc_len=1000),\n    document_filters.ExampleHojiChar()\n])\n```\n```\n>>> cleaner(\'{"text": "こんにちは、"}\')\n\'こんにちは、<hojichar>\'\n```\n上記のフィルタでは 1. JSONから`\'text\'` キーの値を取得 2. 日本語文字列でなければ破棄, 3. 0字以上1000字以内の文章以外を破棄, 4. 文字列に `<hojichar>` を追加 の処理をしています.\n\n各フィルタの処理についてはフィルタの Docstrings に記載されています。\n\n### ユーザー定義フィルタ\n`Filter` クラスを継承し, `apply` 関数内にフィルタの挙動を記述します.\n```Python\nfrom hojichar.core.filter_interface import Filter\n\nclass YourFilter(Filter):\n    def apply(self, document):\n        document.text = your_process(document.text)\n        return document\n```\n`apply` 関数は `hojichar.core.models.Document` 型を引数として受け取り,\n返す関数です. `Document` は文字列をカプセル化したクラスです.\n\n\n## Reference\n*準備中*\n\n各フィルタの処理内容は、`hojichar.filters` モジュール内の Docstrings に記載されています。\n\n\n\n## 開発者向け\n\n開発用のパッケージのインストールのために,\n```\npoetry install --with dev,lint,test\n```\n### テスト\nテスト実行\n```\npoetry run task test\n```\nで mypy と pytest のテストが実行されます.\n\nLint\n```\npoetry run task lint\n```\n\nFormat\n```\npoetry run task format\n```\n',
    'author': 'kenta.shinzato',
    'author_email': 'hoppiece@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/HojiChar/HojiChar',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import json
import logging
import numbers
import pprint
from typing import List, Union

import numpy as np

from hojichar.core.filter_interface import Filter, TokenFilter
from hojichar.core.inspection import Inspector, StatisticsCounter
from hojichar.core.models import Document


class Compose(Filter):
    def __init__(
        self,
        filters: List[Union[Filter, TokenFilter]],
        ignore_filtered: bool = True,
        random_state: Union[None, int, np.random.Generator] = None,
        *args,
        **kwargs,
    ) -> None:
        """
        Compose a filter from pre-defined filter-objects.

        Parameters
        ----------
        filters : List[Union[Filter, TokenFilter]]
            Filter instances which apply to the corpus.

        ignore_filtered : bool, optional
            Default = True
            If True, filters ignore a document which has `is_rejected` flag.
            By doing so, we avoid applying filters that do not affect the output.

            However, for the purpose of corpus survay, e.g., to determine how many documents
            in the corpus are removed by a certain filter, it is preferable not to be
            affected by the upstream filter. In such a case, you should
            set `ignore_filter` to False.

        random_state : Union[None, int, np.random.Generator], optional
            Default = None
            Seed for applying filters randomly.
            `random_state` must be int or np.random.Generator instance.
        """
        super().__init__(*args, **kwargs)
        self.filters = filters
        self.filter_name_list = [f"{i}-{filt.name}" for i, filt in enumerate(self.filters)]
        self.logger = logging.getLogger("hojichar.Compose")
        self.ignore_filtered = ignore_filtered
        self.before_process_inspector = Inspector(
            target="before_process", ignore_filtered=ignore_filtered
        )
        self.inspectors = [
            Inspector(target=name, ignore_filtered=ignore_filtered)
            for name in self.filter_name_list
        ]
        self._statistics = StatisticsCounter(self.inspectors, ignore_filtered=self.ignore_filtered)

        # Turn random_state into a `np.random.Generator` instance.
        if random_state is None:
            self.rng = np.random.default_rng()
        elif isinstance(random_state, numbers.Integral):
            self.rng = np.random.default_rng(random_state)
        elif isinstance(random_state, np.random.Generator):
            self.rng = random_state
        else:
            raise ValueError(f"{random_state} cannot be used to seed.")

    def __call__(self, text: str) -> str:
        document = Document(text)
        document = self.apply(document)
        return document.text

    def apply(self, document: Document) -> Document:
        document = self.before_process_inspector.apply(document)
        for i, filt in enumerate(self.filters):
            if (filt.p == 1) or (filt.p is None):
                document = filt.filter_apply(document)
            else:
                if self.rng.random() < filt.p:
                    document = filt.filter_apply(document)

            # If `ignore_filtered==False`, filters process a doc which is rejected
            # Under the setting, each inspector always re-set `doc.is_rejected` to `False`.
            document = self.inspectors[i].apply(document)

        document.processed_text = document.text
        if sum([inspector.is_rejected for inspector in self.inspectors]):
            document.is_rejected = True
            document.text = ""

        self._statistics.update_changes(document, self.before_process_inspector, self.inspectors)
        return document

    @property
    def statistics(self) -> dict:
        return self._statistics.get_statistics()

    def summary(self, format="print"):
        info = [
            {
                "layer": i,
                "name": filt.name,
                "doc": filt.__doc__,
            }
            for i, filt in enumerate(self.filters)
        ]

        def to_json(filter_info: dict) -> dict:
            filter_info["doc"] = "".join(d.strip() for d in filter_info["doc"].split("\n"))
            return filter_info

        if format == "json":
            print(json.dumps(list(map(to_json, info)), ensure_ascii=False, indent="\t"))
        if format == "print":
            for layer in info:
                print(f"[{layer['layer']}] {layer['name']}")
                pprint.pprint(layer["doc"])
